# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Data Transfer Medium details for the Migration. If not specified, it will default to Database Link. Only one type
  # of data transfer medium can be specified, except for the case of Amazon RDS Oracle as source, where Object Storage
  # Details along with AwsS3Details are required.
  #
  class DatabaseMigration::Models::CreateDataTransferMediumDetails
    # @return [OCI::DatabaseMigration::Models::CreateDatabaseLinkDetails]
    attr_accessor :database_link_details

    # @return [OCI::DatabaseMigration::Models::CreateObjectStoreBucket]
    attr_accessor :object_storage_details

    # @return [OCI::DatabaseMigration::Models::CreateAwsS3Details]
    attr_accessor :aws_s3_details

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'database_link_details': :'databaseLinkDetails',
        'object_storage_details': :'objectStorageDetails',
        'aws_s3_details': :'awsS3Details'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'database_link_details': :'OCI::DatabaseMigration::Models::CreateDatabaseLinkDetails',
        'object_storage_details': :'OCI::DatabaseMigration::Models::CreateObjectStoreBucket',
        'aws_s3_details': :'OCI::DatabaseMigration::Models::CreateAwsS3Details'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [OCI::DatabaseMigration::Models::CreateDatabaseLinkDetails] :database_link_details The value to assign to the {#database_link_details} property
    # @option attributes [OCI::DatabaseMigration::Models::CreateObjectStoreBucket] :object_storage_details The value to assign to the {#object_storage_details} property
    # @option attributes [OCI::DatabaseMigration::Models::CreateAwsS3Details] :aws_s3_details The value to assign to the {#aws_s3_details} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.database_link_details = attributes[:'databaseLinkDetails'] if attributes[:'databaseLinkDetails']

      raise 'You cannot provide both :databaseLinkDetails and :database_link_details' if attributes.key?(:'databaseLinkDetails') && attributes.key?(:'database_link_details')

      self.database_link_details = attributes[:'database_link_details'] if attributes[:'database_link_details']

      self.object_storage_details = attributes[:'objectStorageDetails'] if attributes[:'objectStorageDetails']

      raise 'You cannot provide both :objectStorageDetails and :object_storage_details' if attributes.key?(:'objectStorageDetails') && attributes.key?(:'object_storage_details')

      self.object_storage_details = attributes[:'object_storage_details'] if attributes[:'object_storage_details']

      self.aws_s3_details = attributes[:'awsS3Details'] if attributes[:'awsS3Details']

      raise 'You cannot provide both :awsS3Details and :aws_s3_details' if attributes.key?(:'awsS3Details') && attributes.key?(:'aws_s3_details')

      self.aws_s3_details = attributes[:'aws_s3_details'] if attributes[:'aws_s3_details']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        database_link_details == other.database_link_details &&
        object_storage_details == other.object_storage_details &&
        aws_s3_details == other.aws_s3_details
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [database_link_details, object_storage_details, aws_s3_details].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
