# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

module OCI
  module ServiceMesh
    # Module containing models for requests made to, and responses received from,
    # OCI ServiceMesh services
    module Models
    end
  end
end

# Require models
require 'oci/service_mesh/models/access_logging_configuration'
require 'oci/service_mesh/models/access_policy'
require 'oci/service_mesh/models/access_policy_collection'
require 'oci/service_mesh/models/access_policy_rule'
require 'oci/service_mesh/models/access_policy_summary'
require 'oci/service_mesh/models/access_policy_target'
require 'oci/service_mesh/models/action_type'
require 'oci/service_mesh/models/all_virtual_services_access_policy_target'
require 'oci/service_mesh/models/ca_bundle'
require 'oci/service_mesh/models/certificate_authority'
require 'oci/service_mesh/models/change_access_policy_compartment_details'
require 'oci/service_mesh/models/change_ingress_gateway_compartment_details'
require 'oci/service_mesh/models/change_ingress_gateway_route_table_compartment_details'
require 'oci/service_mesh/models/change_mesh_compartment_details'
require 'oci/service_mesh/models/change_virtual_deployment_compartment_details'
require 'oci/service_mesh/models/change_virtual_service_compartment_details'
require 'oci/service_mesh/models/change_virtual_service_route_table_compartment_details'
require 'oci/service_mesh/models/create_access_policy_details'
require 'oci/service_mesh/models/create_ingress_gateway_details'
require 'oci/service_mesh/models/create_ingress_gateway_mutual_transport_layer_security_details'
require 'oci/service_mesh/models/create_ingress_gateway_route_table_details'
require 'oci/service_mesh/models/create_mesh_details'
require 'oci/service_mesh/models/create_mutual_transport_layer_security_details'
require 'oci/service_mesh/models/create_virtual_deployment_details'
require 'oci/service_mesh/models/create_virtual_service_details'
require 'oci/service_mesh/models/create_virtual_service_route_table_details'
require 'oci/service_mesh/models/default_virtual_service_routing_policy'
require 'oci/service_mesh/models/dns_service_discovery_configuration'
require 'oci/service_mesh/models/external_service_access_policy_target'
require 'oci/service_mesh/models/http_ingress_gateway_traffic_route_rule'
require 'oci/service_mesh/models/http_virtual_service_traffic_route_rule'
require 'oci/service_mesh/models/ingress_gateway'
require 'oci/service_mesh/models/ingress_gateway_access_policy_target'
require 'oci/service_mesh/models/ingress_gateway_collection'
require 'oci/service_mesh/models/ingress_gateway_host'
require 'oci/service_mesh/models/ingress_gateway_host_ref'
require 'oci/service_mesh/models/ingress_gateway_listener'
require 'oci/service_mesh/models/ingress_gateway_mutual_transport_layer_security'
require 'oci/service_mesh/models/ingress_gateway_route_table'
require 'oci/service_mesh/models/ingress_gateway_route_table_collection'
require 'oci/service_mesh/models/ingress_gateway_route_table_summary'
require 'oci/service_mesh/models/ingress_gateway_summary'
require 'oci/service_mesh/models/ingress_gateway_traffic_route_rule'
require 'oci/service_mesh/models/ingress_listener_client_validation_config'
require 'oci/service_mesh/models/ingress_listener_tls_config'
require 'oci/service_mesh/models/local_file_ca_bundle'
require 'oci/service_mesh/models/local_file_tls_certificate'
require 'oci/service_mesh/models/mesh'
require 'oci/service_mesh/models/mesh_collection'
require 'oci/service_mesh/models/mesh_mutual_transport_layer_security'
require 'oci/service_mesh/models/mesh_summary'
require 'oci/service_mesh/models/mutual_transport_layer_security'
require 'oci/service_mesh/models/oci_ca_bundle'
require 'oci/service_mesh/models/oci_tls_certificate'
require 'oci/service_mesh/models/operation_status'
require 'oci/service_mesh/models/operation_type'
require 'oci/service_mesh/models/proxy_details'
require 'oci/service_mesh/models/service_discovery_configuration'
require 'oci/service_mesh/models/sort_order'
require 'oci/service_mesh/models/tcp_ingress_gateway_traffic_route_rule'
require 'oci/service_mesh/models/tcp_virtual_service_traffic_route_rule'
require 'oci/service_mesh/models/tls_certificate'
require 'oci/service_mesh/models/tls_passthrough_ingress_gateway_traffic_route_rule'
require 'oci/service_mesh/models/tls_passthrough_virtual_service_traffic_route_rule'
require 'oci/service_mesh/models/traffic_rule_target'
require 'oci/service_mesh/models/update_access_policy_details'
require 'oci/service_mesh/models/update_ingress_gateway_details'
require 'oci/service_mesh/models/update_ingress_gateway_route_table_details'
require 'oci/service_mesh/models/update_mesh_details'
require 'oci/service_mesh/models/update_virtual_deployment_details'
require 'oci/service_mesh/models/update_virtual_service_details'
require 'oci/service_mesh/models/update_virtual_service_route_table_details'
require 'oci/service_mesh/models/virtual_deployment'
require 'oci/service_mesh/models/virtual_deployment_collection'
require 'oci/service_mesh/models/virtual_deployment_listener'
require 'oci/service_mesh/models/virtual_deployment_summary'
require 'oci/service_mesh/models/virtual_deployment_traffic_rule_target'
require 'oci/service_mesh/models/virtual_service'
require 'oci/service_mesh/models/virtual_service_access_policy_target'
require 'oci/service_mesh/models/virtual_service_collection'
require 'oci/service_mesh/models/virtual_service_route_table'
require 'oci/service_mesh/models/virtual_service_route_table_collection'
require 'oci/service_mesh/models/virtual_service_route_table_summary'
require 'oci/service_mesh/models/virtual_service_summary'
require 'oci/service_mesh/models/virtual_service_traffic_route_rule'
require 'oci/service_mesh/models/virtual_service_traffic_rule_target'
require 'oci/service_mesh/models/work_request'
require 'oci/service_mesh/models/work_request_collection'
require 'oci/service_mesh/models/work_request_error'
require 'oci/service_mesh/models/work_request_error_collection'
require 'oci/service_mesh/models/work_request_log_entry'
require 'oci/service_mesh/models/work_request_log_entry_collection'
require 'oci/service_mesh/models/work_request_resource'
require 'oci/service_mesh/models/work_request_summary'

# Require generated clients
require 'oci/service_mesh/service_mesh_client'
require 'oci/service_mesh/service_mesh_client_composite_operations'

# Require service utilities
require 'oci/service_mesh/util'
