# Copyright (c) 2016, 2022, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Rate limiting configuration.
  class Waf::Models::RequestRateLimitingConfiguration
    # **[Required]** Evaluation period in seconds.
    # @return [Integer]
    attr_accessor :period_in_seconds

    # **[Required]** Requests allowed per evaluation period.
    # @return [Integer]
    attr_accessor :requests_limit

    # Duration of block action application in seconds when `requestsLimit` is reached. Optional and can be 0 (no block duration).
    # @return [Integer]
    attr_accessor :action_duration_in_seconds

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'period_in_seconds': :'periodInSeconds',
        'requests_limit': :'requestsLimit',
        'action_duration_in_seconds': :'actionDurationInSeconds'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'period_in_seconds': :'Integer',
        'requests_limit': :'Integer',
        'action_duration_in_seconds': :'Integer'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [Integer] :period_in_seconds The value to assign to the {#period_in_seconds} property
    # @option attributes [Integer] :requests_limit The value to assign to the {#requests_limit} property
    # @option attributes [Integer] :action_duration_in_seconds The value to assign to the {#action_duration_in_seconds} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.period_in_seconds = attributes[:'periodInSeconds'] if attributes[:'periodInSeconds']

      raise 'You cannot provide both :periodInSeconds and :period_in_seconds' if attributes.key?(:'periodInSeconds') && attributes.key?(:'period_in_seconds')

      self.period_in_seconds = attributes[:'period_in_seconds'] if attributes[:'period_in_seconds']

      self.requests_limit = attributes[:'requestsLimit'] if attributes[:'requestsLimit']

      raise 'You cannot provide both :requestsLimit and :requests_limit' if attributes.key?(:'requestsLimit') && attributes.key?(:'requests_limit')

      self.requests_limit = attributes[:'requests_limit'] if attributes[:'requests_limit']

      self.action_duration_in_seconds = attributes[:'actionDurationInSeconds'] if attributes[:'actionDurationInSeconds']

      raise 'You cannot provide both :actionDurationInSeconds and :action_duration_in_seconds' if attributes.key?(:'actionDurationInSeconds') && attributes.key?(:'action_duration_in_seconds')

      self.action_duration_in_seconds = attributes[:'action_duration_in_seconds'] if attributes[:'action_duration_in_seconds']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        period_in_seconds == other.period_in_seconds &&
        requests_limit == other.requests_limit &&
        action_duration_in_seconds == other.action_duration_in_seconds
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [period_in_seconds, requests_limit, action_duration_in_seconds].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
